# -*- coding: utf-8 -*-

#  PycaWM
#  pager.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('pager module needs at least Python 2.5!')

from Xlib import X, display

from pycawm.hookmanager import add_post_hook, remove_post_hook
from pycawm.wrappers import DisplayWrapper
from pycawm import Client

class Pager(object):
    ratio = 10
    def __init__(self, wm, x=0, y=0):
        self.wm = wm
        self.display = display.Display(self.wm.display.get_display_name())
        self.display_wrapper = DisplayWrapper(self.display, self.handle_events)
        self.wm.displays.append(self.display_wrapper)
        self.screen = self.display.screen()
        self.root = self.screen.root
        self.font = self.display.open_font('fixed')
        root_geo = self.root.get_geometry()
        self.window = self.root.create_window(
            x, y,
            root_geo.width / Pager.ratio, root_geo.height / Pager.ratio,
            2, self.screen.root_depth,
            event_mask=X.ExposureMask|X.ButtonPressMask,
            background_pixel=self.screen.black_pixel,
            override_redirect=True)
        geo = self.window.get_geometry()
        self.window_pixmap = self.window.create_pixmap(geo.width, geo.height,
                                                       self.screen.root_depth)
        self.client_gc = self.window.create_gc(
            background=self.screen.white_pixel,
            font=self.font)
        self.text_gc = self.window.create_gc(
            foreground=self.screen.white_pixel,
            font=self.font)
        add_post_hook(self.wm.handle_events, self.draw_pager)
        self.window.map()
        self.window.configure(stack_mode=X.Below)
        self.draw_pager()
        self.display.sync()

    def destroy(self):
        remove_post_hook(self.wm.handle_events, self.draw_pager)
        try:
            self.wm.displays.remove(self.display_wrapper)
        except ValueError:
            print 'pager socket wrapper not in the connection list!'
        self.window.kill_client()
        self.font.close()
        self.client_gc.free()
        self.text_gc.free()
        self.window_pixmap.free()

    def expose_reply(self, event):
        self.draw_pager()

    def draw_pager(self, *args):
        wm_screen = self.wm.screen
        window_geometry = self.window.get_geometry()
        self.client_gc.change(foreground=wm_screen.black_pixel)
        self.window_pixmap.fill_rectangle(self.client_gc, 0, 0,
                                          window_geometry.width,
                                          window_geometry.height)
        for client in reversed([client for client in \
                                    self.wm.stacked_clients \
                                    if not client.is_hidden()]):
            geometry = client.geometry
            x = geometry.x / Pager.ratio
            y = geometry.y / Pager.ratio
            width = geometry.width / Pager.ratio
            height = geometry.height / Pager.ratio
            if width < 1 or height < 1:
                # client too small to be displayed correctly...
                continue
            self.client_gc.change(foreground=self.wm.get_color(
                    'black' if client.states.focused else 'DimGray'))
            self.window_pixmap.rectangle(self.text_gc, x, y, width, height)
            # can we show the inside rectangle properly?
            if width > 1 and height > 1:
                pixmap_font = self.window_pixmap.create_pixmap(
                    width - 1, height - 1, wm_screen.root_depth)
                pixmap_font.fill_rectangle(self.client_gc, 0, 0,
                                           width - 1, height - 1)
                client_name = str(client.name)
                if client_name:
                    extents = self.font.query_text_extents(client_name)
                    self.text_gc.change(
                        background= self.wm.get_color(
                            'black' if client.states.focused else 'DimGray'))
                    pixmap_font.image_text(self.text_gc,
                                           0, extents.overall_ascent,
                                           client_name)
                    self.window_pixmap.copy_area(self.client_gc, pixmap_font,
                                                 0, 0, width - 1, height - 1,
                                                 x + 1, y + 1)
                    pixmap_font.free()
        self.window.copy_area(self.client_gc, self.window_pixmap,
                              0, 0,
                              window_geometry.width, window_geometry.height,
                              0, 0)
        self.display.sync()

    def button_press_reply(self, event):
        point_x = event.event_x * Pager.ratio
        point_y = event.event_y * Pager.ratio
        client = self.wm.client_at_point(point_x, point_y)
        if client:
            print ('client %s with name "%s" has been selected' %
                   (client, client.name))
            self.wm.give_focus(client)
            self.draw_pager()

    def handle_events(self):
        while self.display.pending_events():
            event = self.display.next_event()
            if event.type == X.Expose:
                self.draw_pager()
            elif event.type == X.ButtonPress:
                self.button_press_reply(event)
